SystemOrganization addCategory: #FuelPackageLoaderTests!

FLSerializationTest subclass: #FLPackageLoaderTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FuelPackageLoaderTests'!

!FLPackageLoaderTest classSide methodsFor: 'accessing' stamp: 'MarianoMartinezPeck 10/31/2011 12:00'!
packageNamesUnderTest
	^ #('FuelPackageLoader')! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/24/2011 11:37'!
categoryForMethodExtensions
	"Answers the category in which tests should add method extensions."
	
	^self stubPackageInfo methodCategoryPrefix! !

!FLPackageLoaderTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 4/19/2012 18:57'!
storeRemoveAndLoadStubPackage

	self streamFactory writeStreamDo: [:aStream |
		FLPackageStore new storeOn: aStream packageNamed: classFactory packageName ].

	"Traits are not correctly cleaned with the classFactory. Therefore, we do the full #tearDown"
	self tearDown.
	self stubPackageInfo extensionMethods do: [ :method | method methodClass removeSelectorSilently: method selector ].
	self assert: self stubPackageInfo classes isEmpty.
	self assert: self stubPackageInfo extensionMethods isEmpty.

	self streamFactory readStreamDo: [:aStream |
		| package |
		SystemChangeNotifier uniqueInstance doSilently: [
			package := FLPackageLoader new loadFrom: aStream.
		].
		classFactory createdClasses: package behaviors ].
	! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 1/24/2012 20:38'!
stubPackageInfo
	^ PackageInfo new packageName: classFactory packageName
! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:07'!
testClassInitialization
	"It should send #initialize to each class."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass class 
		duringTestCompileSilently: 'initialize  ClassVariable := 1';
		duringTestCompileSilently: 'isInitialized  ^ClassVariable isNil not'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.
	
	self assert: materializedClass isInitialized.! !

!FLPackageLoaderTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 4/19/2012 18:57'!
testClassRegistration
	"The class should be included in Smalltalk system dictionary."

	| aClass name materializedClass |
	"In this case we need that the class creation sends notification because of the last asssert."
	aClass := classFactory newClassWithNotifications.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.

	materializedClass := 
		Smalltalk 
			at: name 
			ifAbsent: [ self signalFailure: 'The class should be included in Smalltalk system dictionary.' ].
			
	self assert: classFactory defaultCategory equals: (Smalltalk organization categoryOfElement: materializedClass name).
! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/28/2011 20:22'!
testExternalSuperclass
	"Tests consistency of classes' superclass."

	| aClass name materializedClass |
	aClass := classFactory newClass.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass superclass == aClass superclass.
	self assert: (materializedClass superclass subclasses identityIncludes: materializedClass).! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:07'!
testIgnoreClassSideVariable
	"It should not serialize class side variable values."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass class
		addInstVarNamed: 'classSideVariable';  
		duringTestCompileSilently: 'initializeClassSideVariable  classSideVariable := 1';
		duringTestCompileSilently: 'classSideVariable  ^classSideVariable'.
	aClass initializeClassSideVariable.
	self deny: aClass classSideVariable isNil.
	
	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classSideVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testIgnoreClassVariableWithClassSideMethod
	"It should not serialize class variable values."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass class
		duringTestCompileSilently: 'initializeClassVariable  ClassVariable := 1';
		duringTestCompileSilently: 'classVariable  ^ClassVariable'.
	aClass initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:07'!
testIgnoreClassVariableWithInstanceSideMethod
	"It should not serialize class variable values."

	| aClass name materializedClass |
	aClass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass
		duringTestCompileSilently: 'initializeClassVariable  ClassVariable := 1';
		duringTestCompileSilently: 'classVariable  ^ClassVariable'.
	aClass new initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass new classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:21'!
testIgnoreForeignClassSideMethod
	"It should not serialize class foreign protocol."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass class
		duringTestCompileSilently: 'foreign' classified: '*foreigns';
		duringTestCompileSilently: 'core' classified: 'cores'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: #(core) asSet equals: materializedClass class localSelectors.	
	self assert: (materializedClass class includesSelector: #core).
	self deny: (materializedClass class includesSelector: #foreign).
	self assert: #(cores) equals: materializedClass class organization categories.
	self assert: #(core) equals: materializedClass class organization allMethodSelectors.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:21'!
testIgnoreForeignInstanceSideMethod
	"It should not serialize class foreign protocol."

	| aClass name materializedClass |
	aClass := classFactory newClass.
	aClass
		duringTestCompileSilently: 'foreign' classified: '*foreigns';
		duringTestCompileSilently: 'core' classified: 'cores'.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: #(core) asSet equals: materializedClass localSelectors.	
	self assert: (materializedClass includesSelector: #core).
	self deny: (materializedClass includesSelector: #foreign).
	self assert: #(cores) equals: materializedClass organization categories.
	self assert: #(core) equals: materializedClass organization allMethodSelectors.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testIgnoreSuperClassSideVariable
	"It should not serialize class side variable values even if they come from super."

	| aSuperclass aClass name materializedClass |
	aSuperclass := classFactory newClass.
	aClass := classFactory newSubclassOf: aSuperclass instanceVariableNames: '' classVariableNames: ''.
	
	aSuperclass class
		addInstVarNamed: 'classSideVariable'.
	aClass class
		duringTestCompileSilently: 'initializeClassSideVariable  classSideVariable := 1';
		duringTestCompileSilently: 'classSideVariable  ^classSideVariable'.
		
	aClass initializeClassSideVariable.
	self deny: aClass classSideVariable isNil.
	
	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classSideVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:07'!
testIgnoreSuperClassVariableWithClassSideMethod
	"It should not serialize class variable values even if it comes from super."

	| aSuperclass aClass name materializedClass |
	aSuperclass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass := classFactory newSubclassOf: aSuperclass instanceVariableNames: '' classVariableNames: ''.

	aClass class
		duringTestCompileSilently: 'initializeClassVariable  ClassVariable := 1';
		duringTestCompileSilently: 'classVariable  ^ClassVariable'.
	aClass initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testIgnoreSuperClassVariableWithInstanceSideMethod
	"It should not serialize class variable values."

	| aSuperclass aClass name materializedClass |
	aSuperclass := classFactory newSubclassOf: Object instanceVariableNames: '' classVariableNames: 'ClassVariable'.
	aClass := classFactory newSubclassOf: aSuperclass instanceVariableNames: '' classVariableNames: ''.
	aClass
		duringTestCompileSilently: 'initializeClassVariable  ClassVariable := 1';
		duringTestCompileSilently: 'classVariable  ^ClassVariable'.
	aClass new initializeClassVariable.

	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: materializedClass new classVariable isNil.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 9/28/2011 21:03'!
testInternalSuperclass
	"Tests consistency of classes' superclass."

	| aClass aSuperclass className superclassName materializedClass materializedSuperclass |
	aSuperclass := classFactory newClass.
	aClass := classFactory newSubclassOf: aSuperclass instanceVariableNames: '' classVariableNames: ''.

	superclassName := aSuperclass name.
	className := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedSuperclass := Smalltalk at: superclassName.
	materializedClass := Smalltalk at: className.

	self assert: materializedClass superclass == materializedSuperclass.
	self assert: (materializedClass superclass subclasses includes: materializedClass).
	self assert: materializedSuperclass subclasses = (Array with: materializedClass).! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testMethodWithUndeclaredGlobal

	| name materializedClass aClass |
	Smalltalk globals removeKey: #FLGlobalVariableForTesting ifAbsent: [].
	aClass := self classFactory newClass.
	aClass class duringTestCompileSilently: 'foo  ^ FLGlobalVariableForTesting'.
	
	name := aClass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.

	self assert: (materializedClass class >> #foo valueWithReceiver: aClass arguments: #()) isNil.
	Smalltalk globals at: #FLGlobalVariableForTesting put: 42.
	self assert: (materializedClass class >> #foo valueWithReceiver: aClass arguments: #()) = 42.
! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:21'!
testPackageWithMethodExtension
	"It should serialize package's extension methods."

	| classWithExtension categoryForMethodExtension |
	categoryForMethodExtension := self categoryForMethodExtensions.
	classWithExtension := FLPackageLoaderDummy.
	classWithExtension duringTestCompileSilently: 'fortyTwo ^42' classified: categoryForMethodExtension. 
	
	self storeRemoveAndLoadStubPackage.
	
	self assert: (classWithExtension includesSelector: #fortyTwo).
	self assert: categoryForMethodExtension equals: (classWithExtension whichCategoryIncludesSelector: #fortyTwo).
	self assert: (PackageOrganizer default packageNames includes: classFactory packageName)! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:48'!
testPackageWithMethodExtensionWithClassSideVariable
	"Tests that the extension method is corretly rebinded on load."

	| categoryForMethodExtension |
	categoryForMethodExtension := self categoryForMethodExtensions.
	FLPackageLoaderDummy class 
		duringTestCompileSilently: 'initializeFortyTwo  classInstanceVariable := 42' 
		classified: categoryForMethodExtension. 
	FLPackageLoaderDummy classInstanceVariable: nil.

	self storeRemoveAndLoadStubPackage.

	"Bindings in the class method should be recalculated on load."	
	self shouldnt: [ FLPackageLoaderDummy initializeFortyTwo ] raise: Error.
	self assert: 42 equals: FLPackageLoaderDummy classInstanceVariable.

! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:21'!
testPackageWithMethodExtensionWithClassVariable
	"Tests that the extension method is corretly rebinded on load."

	| categoryForMethodExtension |
	categoryForMethodExtension := self categoryForMethodExtensions.
	FLPackageLoaderDummy class 
		duringTestCompileSilently: 'initializeFortyTwo  ClassVariable := 42' 
		classified: categoryForMethodExtension. 

	"After next lines, the class method points to an old binding. (This is not the ideal way of testing because it is breaking consistency of the class.)"
	FLPackageLoaderDummy classPool removeKey: #ClassVariable.
	FLPackageLoaderDummy classPool at: #ClassVariable put: nil.

	self storeRemoveAndLoadStubPackage.

	"Bindings in the class method should be recalculated on load."	
	self shouldnt: [ FLPackageLoaderDummy initializeFortyTwo ] raise: Error.
	self assert: 42 equals: (FLPackageLoaderDummy classPool at: #ClassVariable).

! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MartinDias 8/21/2011 22:04'!
testPackageWithTwoCategories
	"It should serialize multiple categories by package."

	| aStream classNames |
	classFactory newClassInCategory: 'One'.
	classFactory newClassInCategory: 'Two'.
	classNames := classFactory createdClassNames.

	self storeRemoveAndLoadStubPackage.
	
	self assert: (PackageOrganizer default packageNames includes: classFactory packageName) description: 'The package should be present in PackageOrganizer default.'.! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 9/29/2011 19:21'!
testSubclassesRegistration
	"Tests consistency of classes' superclass."

	| aClass name materializedClass aSubclass subclassName materializeSubclass |
	aClass := classFactory newClass.
	aSubclass := classFactory newSubclassOf: aClass instanceVariableNames: '' classVariableNames:  ''. 

	name := aClass name.
	subclassName := aSubclass name.
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.
	materializeSubclass := Smalltalk at: subclassName.

	self assert: materializedClass superclass == aClass superclass.
	self assert: (materializedClass superclass subclasses identityIncludes: materializedClass).
	self assert: (materializeSubclass superclass == materializedClass).
	self assert: (materializedClass subclasses identityIncludes: materializeSubclass).
	! !

!FLPackageLoaderTest methodsFor: 'tests' stamp: 'MarianoMartinezPeck 4/19/2012 12:46'!
testTraitLoad
	"Tests that traits are correctly reinserted in the system."

	| aClass name materializedClass  materializedTrait aTrait traitName |
	aClass := classFactory newClass.
	aTrait := self newTraitSuffixed: 'aTrait'.
	name := aClass name.
	aTrait duringTestCompileSilently: 'instanceBindings  Object name. ', name , '  name.'.
	aClass addToComposition: aTrait.
	traitName := aTrait name.
	
	self storeRemoveAndLoadStubPackage.
	materializedClass := Smalltalk at: name.
	materializedTrait := Smalltalk at: traitName.

	self assert: (materializedClass traitComposition includesTrait: materializedTrait).
	self assert: ((materializedTrait >> #instanceBindings) literalAt: 2) == (Smalltalk globals associationAt: #Object).
	self assert: ((materializedTrait >> #instanceBindings) literalAt: 3) == (Smalltalk globals associationAt: name).
	! !

Object subclass: #FLPackageLoaderDummy
	instanceVariableNames: ''
	classVariableNames: 'ClassVariable'
	poolDictionaries: ''
	category: 'FuelPackageLoaderTests'!

FLPackageLoaderDummy class
	instanceVariableNames: 'classInstanceVariable'!

FLPackageLoaderDummy class
	instanceVariableNames: 'classInstanceVariable'!

!FLPackageLoaderDummy classSide methodsFor: 'accessing' stamp: 'MartinDias 9/24/2011 11:38'!
classInstanceVariable
	^ classInstanceVariable! !

!FLPackageLoaderDummy classSide methodsFor: 'accessing' stamp: 'MartinDias 9/24/2011 11:38'!
classInstanceVariable: anObject
	classInstanceVariable := anObject! !
